# Copyright (c) 2018 Stephen Bunn <stephen@bunn.io>
# MIT License <https://opensource.org/licenses/MIT>

from ._common import BaseHandler


class JSONHandler(BaseHandler):

    name = "json"
    packages = ("ujson", "json")

    def on_json_dumps(self, json, dictionary):
        return json.dumps(dictionary)

    def on_json_loads(self, json, content):
        return json.loads(content)

    def on_ujson_dumps(self, ujson, dictionary):
        return ujson.dumps(dictionary)

    def on_ujson_loads(self, ujson, content):
        return ujson.loads(content)



# Copyright (c) 2018 Stephen Bunn <stephen@bunn.io>
# MIT License <https://opensource.org/licenses/MIT>

from collections import OrderedDict

from ._common import BaseHandler


class YAMLHandler(BaseHandler):

    name = "yaml"
    packages = ("yaml",)

    def on_yaml_imported(self, yaml):
        def represent_ordereddict(dumper, data):
            """ A custom data representer for ``OrderedDict`` instances.

            .. note:: Credit to https://stackoverflow.com/a/16782282/7828560.

            :param dumper: The dumper object
            :type dumper: object
            :param data: The ``OrderedDict`` instance
            :type data: OrderedDict
            :return: The yaml mapping node
            :rtype: yaml.nodes.MappingNode
            """

            values = []
            for (key, value) in data.items():
                values.append(
                    (dumper.represent_data(key), dumper.represent_data(value))
                )
            return yaml.MappingNode("tag:yaml.org,2002:map", values)

        yaml.add_representer(OrderedDict, represent_ordereddict)

    def on_yaml_dumps(self, yaml, instance):
        return yaml.dump(instance)

    def on_yaml_loads(self, yaml, content):
        return yaml.load(content)



# Copyright (c) 2018 Stephen Bunn <stephen@bunn.io>
# MIT License <https://opensource.org/licenses/MIT>

import abc
import sys
import importlib


class BaseHandler(abc.ABC):
    @property
    def imported(self):
        if not hasattr(self, "_imported"):
            self._imported = self._discover_import()
        return self._imported

    @property
    def handler(self):
        if not hasattr(self, "_handler"):
            self._handler = sys.modules[self.imported]
        return self._handler

    @abc.abstractproperty
    def name(self):
        raise NotImplementedError(f"subclasses must implement 'name'")

    @abc.abstractproperty
    def packages(self):
        raise NotImplementedError(f"subclasses must implement 'packages'")

    @classmethod
    def available(self):
        for module_name in self.packages:
            if importlib.util.find_spec(module_name):
                return True
        return False

    def _discover_import(self):
        for module_name in self.packages:
            spec = importlib.util.find_spec(module_name)
            if spec is not None:
                importlib.import_module(module_name)
                imported_hook = getattr(self, f"on_{module_name}_imported", None)
                if callable(imported_hook):
                    imported_hook(sys.modules[module_name])
                return module_name
        raise ModuleNotFoundError(f"no modules in {self.packages!r} found")

    def dumps(self, instance):
        dumps_hook_name = f"on_{self.imported}_dumps"
        dumps_hook = getattr(self, dumps_hook_name, None)
        if not callable(dumps_hook):
            raise ValueError(
                f"no dumps handler for {self.imported!r}, requires method "
                f"{dumps_hook_name!r} in {self!r}"
            )
        return dumps_hook(self.handler, instance)

    def loads(self, content):
        loads_hook_name = f"on_{self.imported}_loads"
        loads_hook = getattr(self, loads_hook_name, None)
        if not callable(loads_hook):
            raise ValueError(
                f"no loads handler for {self.imported!r}, requires method "
                f"{loads_hook_name!r} in {self!r}"
            )
        return loads_hook(self.handler, content)

    def dump(self, instance, file_object):
        file_object.write(self.dumps(instance))

    def load(self, file_object):
        return self.loads(file_object.read())



# Copyright (c) 2018 Stephen Bunn <stephen@bunn.io>
# MIT License <https://opensource.org/licenses/MIT>

from ._common import BaseHandler


class MsgpackHandler(BaseHandler):

    name = "msgpack"
    packages = ("msgpack",)

    def on_msgpack_dumps(self, msgpack, dictionary):
        return msgpack.dumps(dictionary)

    def on_msgpack_loads(self, msgpack, content):
        return msgpack.loads(content, raw=False)



# Copyright (c) 2018 Stephen Bunn <stephen@bunn.io>
# MIT License <https://opensource.org/licenses/MIT>

from ._common import BaseHandler


class PickleHandler(BaseHandler):

    name = "pickle"
    packages = ("pickle",)

    def on_pickle_dumps(self, pickle, dictionary):
        return pickle.dumps(dictionary, protocol=pickle.HIGHEST_PROTOCOL)

    def on_pickle_loads(self, pickle, content):
        return pickle.loads(content)



# Copyright (c) 2018 Stephen Bunn <stephen@bunn.io>
# MIT License <https://opensource.org/licenses/MIT>

from ._common import BaseHandler


class TOMLHandler(BaseHandler):

    name = "toml"
    packages = ("tomlkit",)

    def on_tomlkit_dumps(self, tomlkit, dictionary):
        return tomlkit.dumps(dictionary)

    def on_tomlkit_loads(self, tomlkit, content):
        return tomlkit.parse(content)
